/**
 * @brief Este projeto demonstra a programacao de um LCD 2x16
 * @author Wu Shin-Ting
 * @date 13/10/2020
 */

#include "derivative.h" /* include peripheral declarations */

#define GPIO_PIN(x) (1<<x)

/**
 * @brief Funcao do byte
 */
typedef enum lcd_RS_tag {
	COMANDO,  /**< Comando */
	DADO	  /**< Dado */
} lcd_RS_type;


/**
 * @brief atraso em numero de interacoes <i>
 * @param[in] i numero de iteracoes
 */
void delay (uint32_t i)
{
	__asm("outerLoop:;"
				"sub %0, %0, #1;"
				"mov r2, #11;"
			"innerLoop:;"
				"sub r2, r2, #1;"
				"cmp r2, #0;"
				"bne innerLoop;"
				"cmp %0, #0;"
				"bne outerLoop;"
			: 
			: "r" (i)
			);
}

/**
 * @brief Inicializa os pinos conectados ao latch 74573 onde estao conectados os 8 leds vermelhos
 */
void GPIO_initLCD(){
	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK ;     ///< Habilita o clock da PORTA C

	// Configura os pinos PORTC[10:0] como GPIO de saida 
	PORTC_PCR0 |= PORT_PCR_MUX(0x1);       ///< D0-D7 dos dados (GPIO)
	PORTC_PCR1 |= PORT_PCR_MUX(0x1);
	PORTC_PCR2 |= PORT_PCR_MUX(0x1);
	PORTC_PCR3 |= PORT_PCR_MUX(0x1);
	PORTC_PCR4 |= PORT_PCR_MUX(0x1);
	PORTC_PCR5 |= PORT_PCR_MUX(0x1);
	PORTC_PCR6 |= PORT_PCR_MUX(0x1);
	PORTC_PCR7 |= PORT_PCR_MUX(0x1);
	
	PORTC_PCR8 |= PORT_PCR_MUX(0x1);       ///< RS do LCD
	PORTC_PCR9 |= PORT_PCR_MUX(0x1);       ///< E do LCD

	GPIOC_PDDR |= GPIO_PDDR_PDD(GPIO_PIN(9)|GPIO_PIN(8) |GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0));       ///< Configura como saidas
	GPIOC_PDOR &= ~GPIO_PDOR_PDO(GPIO_PIN(9) |GPIO_PIN(8)|GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0)); ///< Inicializa os pinos com 0
}

/**
 * @brief Escrever nos pinos de dados um caractere de 8 bits
 * @param[in] c caractere
 */
void GPIO_setByte(char c) {
	// colocar os dados nos pinos de dados DO-D7
	GPIOC_PDOR &= 0xFFFFFF00; 		// zerar o byte menos significativo
	GPIOC_PDOR |= (unsigned int) c; // setas os bits em 1 no byte menos significativo
	
}

/**
 * @brief Gerar o pulso de Enable de ~450ns
 */
void GPIO_enableE()
{
	// gerar um pulso  (~ 450 ns)
	GPIOC_PSOR = GPIO_PIN (9);		// setar em 1
	delay(1);
	GPIOC_PCOR = GPIO_PIN (9);		// resetar em 0
}

/**
 * @brief Setar o tipo de transferencia
 * @param[in] i tipo
 */
void GPIO_setRS (lcd_RS_type i) {
	if(i == COMANDO) 
		GPIOC_PCOR = GPIO_PIN(8);     ///< Seta o LCD no modo de comando		
	else if (i == DADO) 
		GPIOC_PSOR = GPIO_PIN(8);    ///< Seta o LCD no modo de dados
}

/**
 * @brief Transferir um byte ao LCD
 * @param[in] c byte
 * @param[in] t tempo de processamento em multiplos de 2 us
 */
void escreveLCD(char c, uint32_t t)
{
	GPIO_setByte(c);
	GPIO_enableE();
	delay(t);
}

/**
 * @brief Inicializar o LCD
 */
void initLCD () {
	//espera por mais de 30ms = 30000us/2 = 15000
	delay(15000);
	
	GPIO_setRS(COMANDO);
	escreveLCD(0x38, 20);			//Function Set: 39us/2 -> 20
	escreveLCD(0x0C, 20);		    //Display ON/OFF Control: 39us/2 -> 20
	escreveLCD(0x01, 765);			//Display Clear: 1530us/2 -> 765
	escreveLCD(0x06, 20);			//Entry mode set: 39us/2 -> 20
}

/**
 * @brief Escrever uma mensagem str no LCD a partir do endereco especificado
 * @param[in] end endereco DDRAM
 * @param[in] str mensagem
 */
void escreveMensagem (uint8_t end, char* str)
{
	//Posicione o cursor no end
	uint8_t tmp = 0b10000000 | end; // seta o bit 7 em 1
	GPIO_setRS(COMANDO);			// comando
	escreveLCD(tmp, 20);			// seta o endereco do cursor

	//Chaveia para modo DADO
	GPIO_setRS (DADO);
	while (*str) {
		escreveLCD(*str, 20);
		str++;
	}
}

int main(void)
{
	char str1[]="EA 871";

	char lista_cores [6][17] = {"    VERMELHO    ",   //8 caracteres
							    "      VERDE     ",	  //5 caracteres
								"      AZUL      ",	  //4 caracteres
								"     AMARELO    ",	  //7 caracteres
								"      CIANO     ",	  //5 caracteres
								"     MAGENTA    "};	  //7 caracteres

	GPIO_initLCD();
	
	initLCD();
	
	escreveMensagem (0x05, str1);
	
	uint8_t i;
	for (;;) {
		for (i=0; i<6; i++ ) {
			escreveMensagem (0x40, lista_cores[i]);
			delay(500000);
		}
	}
	return 0;
}
